
# q-expansion of newform 2464.2.a.g, downloaded from the LMFDB on 11 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2464
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [0], [1], [-1], [-2], [4], [2], [0], [2], [-10], [10], [-4], [-8], [-6], [-2], [4], [-10], [-12], [-4], [4], [0], [-14], [-14], [2], [18], [-14], [-8], [14], [-18], [16], [6], [-10], [-2], [6], [0], [4], [4], [-12], [6], [4], [16], [-24], [-22], [-22], [2], [-16], [14], [-18], [16], [14], [-24], [-12], [-20], [18], [8], [24], [12], [6], [-10], [30], [-10], [22], [-18], [14], [6], [20], [-18], [-32], [-22], [26], [8], [-2], [26], [20], [22], [10], [20], [14], [8], [-16], [14], [32], [-22], [-20], [4], [-18], [-22], [-14], [20], [0], [32], [8], [20], [4], [28], [0], [18], [14], [10], [12], [-30], [-2], [-18], [28], [46], [36], [36], [-4], [-8], [-8], [-38], [-14], [-8], [-16], [10], [-44], [-2], [-22], [0], [8], [46], [-18], [-36], [-12], [-50], [18], [-18], [22], [-10], [12], [-16], [40], [2], [40], [40], [4], [-6], [12], [-6], [-26], [-10], [40], [4], [48], [-18], [-58], [24], [44], [32], [22], [-18], [28], [-48], [-44], [-48], [-24], [18], [-36], [-18], [-12], [-14], [-32], [-24], [34], [-6], [-44], [18], [-38], [0], [-40], [-28], [-24], [-58], [-14], [54], [-12], [58], [36], [10], [-44], [-42], [34], [-56], [-4], [24], [-14], [-48], [-10], [-12], [-34], [16], [-20], [-20], [-12], [10], [46], [-14], [12], [-60], [-50], [28], [52], [4], [-64], [-48], [-14], [36], [-14], [56], [-50], [-60], [52], [52], [-44], [46], [32], [48], [12], [2], [-6], [52], [-42], [4], [-2], [-18], [22], [-68], [-20], [0], [34], [2], [-40], [-30], [24], [-4], [42], [-42], [-68], [-22], [-44], [-38], [-6], [-34], [56], [-60], [34], [-12], [-12], [-32], [50], [20], [60], [-62], [14], [0], [-14], [22], [32], [28], [16], [12], [-50], [-14], [-18], [60], [6], [14], [-44], [-6], [-52], [-44], [54], [-16], [-52], [70], [-12], [10], [-22], [34], [24], [-4], [48], [-14], [-22], [-48], [-26], [-42], [42], [-24], [-66], [-24], [14], [-70], [46], [18], [-30], [32], [70], [-18], [-8], [-42], [64], [68], [44], [-34], [44], [40], [-44], [6], [32], [42], [-2], [84], [48], [-20], [-20], [-80], [6], [-10], [-38], [-22], [-18], [32], [-6], [-18], [-68], [-48], [8], [-26], [18], [90], [44], [54], [-44], [-24], [50], [-8], [-6], [-48], [-26], [50], [22], [32], [-30], [60], [-56], [52], [0], [12], [-18], [-58], [22], [48], [-42], [-16], [90], [78], [18], [-44], [38], [-24], [76], [76], [-16], [18], [-18], [56], [82], [-84], [64], [56], [-22], [-88], [6], [-32], [-32], [-56], [-36], [-28], [-4], [-54], [32], [18], [22], [-26], [-40], [-48], [64], [58], [-70], [-28], [44], [68], [-28], [54], [64], [30], [4], [54], [30], [-62], [48], [-74], [-50], [28], [38], [18], [-80], [-80], [-84], [10], [-92], [4], [-2], [-84], [-24], [-66], [-58], [104], [-38], [-42], [-92], [-30], [14], [84], [36], [14], [88], [-16]]
